/**********************************************************************************/
/*                 How to Kill Your Server in 60 (75) Minutes                     */
/*                     Written by: Dmitri V. Korotkevitch                         */
/*                            dk@aboutsqlserver.com                               */
/*                          http://aboutsqlserver.com                             */
/**********************************************************************************/
/*                           01. Multi-Table Search                               */
/**********************************************************************************/

set nocount on
set xact_abort on
go

use ForestSoft
go

set statistics time, io on
exec demo.OriginalSearch @StartDate='2010-01-01', @StopDate='2014-05-30', @Search='ABAAAE'
exec demo.OriginalSearchV2 @StartDate='2010-01-01', @StopDate='2014-05-30', @Search='ABAAAE'
set statistics time, io off
go



alter table demo.Documents add SearchStr varchar(800)
go

update d
set
	d.SearchStr = d.DocNumber + ' ' + d.DocTitle + ' ' + d.Reference + 
		' ' + IsNull(i.Correspondent,'') + ' ' + IsNull(o.FileNum,'') +
		' ' + IsNull(o.Sender,'') + ' ' + IsNull(dir.Signer,'') + 
		' ' + IsNull(dir.Sender,'') + ' ' + IsNull(dir.Department,'')
from 
	demo.Documents d left outer join demo.IncomingDocuments i on 
		d.DocId = i.DocId and d.DocType = 16
	left outer join demo.OutgoingDocuments o on
		d.DocId = o.DocId and d.DocType = 17
	left outer join demo.DirectiveDocuments dir on
		d.DocId = dir.DocId and d.DocType = 18
go

create index IDX_Documents_DocDate_SearchStr
on demo.Documents(DocDate)
include(SearchStr)
go


create proc demo.NewSearch
(
	@StartDate datetime
	,@StopDate datetime
	,@Search varchar(128)
)
as
	select @Search = '%' + @Search + '%'

	select top 50
		d.DocId, d.DocType, d.DocNumber, d.DocDate, d.DocTitle, d.Reference
	from 
		demo.Documents d 
	where
		d.DocDate between @StartDate and @StopDate and 
		d.SearchStr like @Search
	order by
		d.DocDate, d.DocNumber
go	
			
set statistics io, time on
exec demo.OriginalSearch @StartDate='2010-01-01', @StopDate='2014-05-30', @Search='ABAAAE'
exec demo.NewSearch @StartDate='2010-01-01', @StopDate='2014-05-30', @Search='ABAAAE'
set statistics io, time off
go


alter table demo.Documents add SearchStrBin varchar(800) collate Latin1_General_100_BIN2
go

update demo.Documents
set	SearchStrBin = upper(SearchStr) collate Latin1_General_100_BIN2;

create index IDX_Documents_DocDate_SearchStrBin
on demo.Documents(DocDate)
include(SearchStrBin)
go

create proc demo.NewSearchV2
(
	@StartDate datetime
	,@StopDate datetime
	,@Search varchar(128)
)
as
	select @Search = '%' + upper(@Search) + '%'

	select top 50
		d.DocId, d.DocType, d.DocNumber, d.DocDate, d.DocTitle, d.Reference
	from 
		demo.Documents d
	where
		d.DocDate between @StartDate and @StopDate and 
		d.SearchStrBin like @Search collate Latin1_General_100_BIN2
	order by
		d.DocDate, d.DocNumber
go	
			
set statistics time on
exec demo.OriginalSearch @StartDate='2010-01-01', @StopDate='2014-05-30', @Search='ABAAAE'
exec demo.NewSearch @StartDate='2010-01-01', @StopDate='2014-05-30', @Search='ABAAAE'
exec demo.NewSearchV2 @StartDate='2010-01-01', @StopDate='2014-05-30', @Search='ABAAAE'
set statistics time off
go


