/**********************************************************************************/
/*                 How to Kill Your Server in 60 (75) Minutes                     */
/*                     Written by: Dmitri V. Korotkevitch                         */
/*                            dk@aboutsqlserver.com                               */
/*                          http://aboutsqlserver.com                             */
/**********************************************************************************/
/*                          02.Paging (Removing Joins)                            */
/**********************************************************************************/

set nocount on
go

use ForestSoft
go

declare
	@Name nvarchar(64) = 'F' /* R */
	,@Number nvarchar(16) = null
	,@SortColumn varchar(128) = 'ProductName'
	,@SortDir varchar(4) = 'ASC'
	,@PageNum int = 2
	,@RecordsPerPage int = 10
	,@TotalRows int

declare
	@SQL nvarchar(max)
	,@Start int = (@PageNum - 1) * @RecordsPerPage + 1
	,@Stop int = @PageNum * @RecordsPerPage

create table #Temp
(
	RowNum int not null,
	ProductId int not null,

	primary key(RowNum)
)

select @SQL = N'
;with CTE
as
(
	select 
		p.ProductId, ROW_NUMBER() over (order by ' + @SortColumn + ' ' + @SortDir + N') as RowNum
	from 
		fs.Products p 
	where 
		1 = 1 ' + 
	case when @Name is not null then N'and (p.ProductName like @Name + ''%'')' else N'' end +
	case when @Number is not null then N' and (p.ProductNumber like @Number + ''%'')' else N'' end + N'
)
insert into #Temp(RowNum, ProductId)
	select RowNum, ProductId
	from CTE;
	
select @TotalRows = @@rowcount'

	--select @SQL for xml path('')

exec sp_executesql
	@SQL
	,N'@Name nvarchar(64), @Number nvarchar(16), @TotalRows int output'
	,@Name = @Name, @Number = @Number, @TotalRows = @TotalRows output

select t.ProductId, p.ProductNumber, p.ProductName, p.Weight, p.DaysToManufacture, m.MaterialName, cat.ProductCategoryName
from 
	#Temp t join fs.Products p on 
		t.ProductId = p.ProductId
	join fs.ProductCategories cat on
		p.ProductCategoryId = cat.ProductCategoryId
	left outer join fs.Materials m on
		p.MaterialId = m.MaterialId
where RowNum between @Start and @Stop
order by RowNum

drop table #Temp
go
