/**********************************************************************************/
/*                 How to Kill Your Server in 60 (75) Minutes                     */
/*                     Written by: Dmitri V. Korotkevitch                         */
/*                            dk@aboutsqlserver.com                               */
/*                          http://aboutsqlserver.com                             */
/**********************************************************************************/
/*                       02.Paging (Removing Temp Table)                          */
/**********************************************************************************/

set nocount on
go

use ForestSoft
go

declare
	@Name nvarchar(64) = 'F' /* R */
	,@Number nvarchar(16) = null
	,@SortColumn varchar(128) = 'ProductName'
	,@SortDir varchar(4) = 'ASC'
	,@PageNum int = 2
	,@RecordsPerPage int = 10
	,@TotalRows int

declare
	@SQL nvarchar(max)
	,@Start int = (@PageNum - 1) * @RecordsPerPage + 1
	,@Stop int = @PageNum * @RecordsPerPage

select @SQL = N'
;with CTE
as
(
	select 
		p.ProductId, ROW_NUMBER() over (order by ' + @SortColumn + ' ' + @SortDir + N') as RowNum
	from 
		fs.Products p 
	where 
		1 = 1 ' + 
	case when @Name is not null then N'and (p.ProductName like @Name + ''%'')' else N'' end +
	case when @Number is not null then N' and (p.ProductNumber like @Number + ''%'')' else N'' end + N'
)
,RowCnt(RowCnt)
as
(
	select count(*) from CTE
) 
select r.RowCnt, t.ProductId, p.ProductNumber, p.ProductName, p.Weight, p.DaysToManufacture, m.MaterialName, cat.ProductCategoryName
from 
	CTE t join fs.Products p on 
		t.ProductId = p.ProductId
	join fs.ProductCategories cat on
		p.ProductCategoryId = cat.ProductCategoryId
	left outer join fs.Materials m on
		p.MaterialId = m.MaterialId
	cross join RowCnt r
where 
	t.RowNum between @Start and @Stop
order by
	t.RowNum'

	--select @SQL for xml path('')

exec sp_executesql
	@SQL
	,N'@Name nvarchar(64), @Number nvarchar(16), @Start int, @Stop int'
	,@Name = @Name, @Number = @Number, @Start = @Start, @Stop = @Stop
go

