/**********************************************************************************/
/*                 How to Kill Your Server in 60 (75) Minutes                     */
/*                     Written by: Dmitri V. Korotkevitch                         */
/*                            dk@aboutsqlserver.com                               */
/*                          http://aboutsqlserver.com                             */
/**********************************************************************************/
/*                             06.Name-Value Pairs                                */
/**********************************************************************************/

set nocount on
go

use ForestSoft
go


create table fs.ProductAttributes
(
	ProductId int not null,
	Attribute varchar(32) not null,
	Value sql_variant not null,

	constraint PK_ProductAttributes
	primary key clustered(ProductId, Attribute)
)
go

insert into fs.ProductAttributes(ProductId, Attribute, Value)
	select ProductId, 'Weight', Weight 
	from fs.Products
	where Weight is not null
	
insert into fs.ProductAttributes(ProductId, Attribute, Value)
	select ProductId, 'Color', Color 
	from fs.Products
	where Color is not null

insert into fs.ProductAttributes(ProductId, Attribute, Value)
	select ProductId, 'DaysToManufacture', DaysToManufacture 
	from fs.Products
	where DaysToManufacture is not null
go

;with Attributes(ProductId, Weight, Color, DaysToManufacture)
as
(
	select 
		ProductId
		,convert(decimal(8,2),[Weight]) as [Weight]
		,convert(nvarchar(15),[Color]) as [Color]
		,convert(int,[DaysToManufacture]) as [DaysToManufacture]
	from
	(
		select ProductId, Attribute, Value
		from fs.ProductAttributes
	) as Source
	pivot (min(Value) for Attribute in ([Weight],[Color],[DaysToManufacture])) as p 
)
--select * from Attributes where ProductId = 600
select p.ProductId, p.ProductNumber, p.ProductName, a.Weight, a.Color, a.DaysToManufacture 
from 
	fs.Products p join Attributes a on
		p.ProductId = a.ProductId
where 
	p.ProductNumber like 'BC%'
go



;with Paging(ProductId, RowNum)
as
(
	select ProductId, ROW_NUMBER() over (order by Value, ProductId)
	from fs.ProductAttributes
	where Attribute = 'DaysToManufacture'
)
,Attributes(ProductId, Weight, Color, DaysToManufacture)
as
(
	select 
		ProductId
		,convert(decimal(8,2),[Weight]) as [Weight]
		,convert(nvarchar(15),[Color]) as [Color]
		,convert(int,[DaysToManufacture]) as [DaysToManufacture]
	from
	(
		select pa.ProductId, pa.Attribute, pa.Value
		from fs.ProductAttributes pa join Paging p on 
			pa.ProductId = p.ProductId
		where
			p.RowNum between 5 and 10
	) as Source
	pivot (min(Value) for Attribute in ([Weight],[Color],[DaysToManufacture])) as p 
)
select p.ProductId, p.ProductNumber, p.ProductName, a.Weight, a.Color, a.DaysToManufacture 
from 
	fs.Products p join Attributes a on
		p.ProductId = a.ProductId

go

